@@ -1,5 +1,5 @@
 use clap::Parser;
-use eyre::{bail, Result};
+use eyre::{Result, bail};
 
 use atuin_client::{api_client, settings::Settings};
 use rpassword::prompt_password;
@@ -15,14 +15,14 @@ pub struct Cmd {
 
 impl Cmd {
     pub async fn run(self, settings: &Settings) -> Result<()> {
-        run(settings, &self.current_password, &self.new_password).await
+        run(settings, self.current_password, self.new_password).await
     }
 }
 
 pub async fn run(
     settings: &Settings,
-    current_password: &Option<String>,
-    new_password: &Option<String>,
+    current_password: Option<String>,
+    new_password: Option<String>,
 ) -> Result<()> {
     let client = api_client::Client::new(
         &settings.sync_address,
